package com.cogitarecomputing.energydefender;
import com.cogitarecomputing.energydefender.Enemy.EnemyTypes;
import flash.display.Bitmap;
import flash.display.Sprite;
import flash.events.MouseEvent;
import openfl.Assets;
import com.cogitarecomputing.energydefender.Shot.ShotType;
import flash.display.BlendMode;

enum EnemyTypes
{
	StraightAttackAndExplodeAtEnergySource;
	StraightAttackAndExplodeAtTower;
	FireAtEnergySource;
	FireAtTower;
}
	
/**
 * ...
 * @author 
 */
class Enemy extends UpdateableSprite
{
	private var bmp:Bitmap;
	public var energy:Float;
	private var m_speed:Float;
	private var m_enemyType:EnemyTypes;
	private var m_main:Main;
	private var m_energySource:EnergySource;
	private var m_shotPulse:Int;
	private var m_lighting:Bitmap;
	private var m_sprite:Sprite;
	public override function Update(deltaTime:Float):Void
	{
		var rot = deltaTime;
		if (m_enemyType == EnemyTypes.StraightAttackAndExplodeAtTower)
			rot = deltaTime * 2;
		if (m_enemyType == EnemyTypes.StraightAttackAndExplodeAtEnergySource)
			rot = -2 * deltaTime;
		if (m_enemyType == EnemyTypes.FireAtEnergySource)
			rot = -deltaTime;
			
		rotation -= rot / 10;
		m_sprite.rotation += rot / 5;
		
		if (energy <= 0.0 && visible)
		{
			visible = false;
			m_main.m_explosionHandler.AddExplosion(x, y, true);
			m_main.shipExplodeSnd.play();
			m_main.m_tower.AddScore(2);
			return;
		}
		
		if (m_enemyType == EnemyTypes.StraightAttackAndExplodeAtTower
		|| m_enemyType == EnemyTypes.FireAtTower)
		{
			var dx:Float = x - (m_main.m_tower.x);
			var dy:Float = y - (m_main.m_tower.y);
			var d:Float = Math.sqrt(dx * dx + dy * dy);
			if (m_shotPulse == 0)
			{
				if (m_enemyType == EnemyTypes.FireAtTower)
				{
					m_main.enemyShotSnd.play();
					m_main.m_shotHandler.Shoot(0.01, 2.0, x, y, -3.0*dx / d, -3.0*dy / d, ShotType.AimAtTower);
				}			
				m_shotPulse = 100;
			}
			else
			{
				m_shotPulse--;
			}
			
			if (d < 50)
			{
				if (m_enemyType != EnemyTypes.FireAtTower)				
				{
					visible = false;
					m_main.m_explosionHandler.AddExplosion(x, y, true);					
					m_main.m_tower.energy -= 0.2;
					m_main.shipExplodeSnd.play();
				}
			}
			else
			{
				dx /= d;
				dy /= d;
				x -= m_speed*dx;
				y -= m_speed*dy;
			}
		}		
		if (m_enemyType == EnemyTypes.StraightAttackAndExplodeAtEnergySource
		|| m_enemyType == EnemyTypes.FireAtEnergySource)
		{			
			if (m_energySource == null || m_energySource.visible == false)
			{
				m_energySource = m_main.findClosestEnergySource(x, y);
			}
			if (m_energySource != null)
			{
				var dx:Float = x - (m_energySource.x);
				var dy:Float = y - (m_energySource.y);
				var d:Float = Math.sqrt(dx * dx + dy * dy);
				if (m_shotPulse == 0)
				{
					if (m_enemyType == EnemyTypes.FireAtEnergySource)
					{
						m_main.enemyShotSnd.play();
						m_main.m_shotHandler.Shoot(0.05, 2.0, x, y, -3.0*dx / d, -3.0*dy / d, ShotType.AimAtEnergy);
					}			
					m_shotPulse = 100;
				}
				else
				{
					m_shotPulse--;
				}

				if (d < 50)
				{
					if (m_enemyType != EnemyTypes.FireAtEnergySource)
					{
						visible = false;
						m_main.m_explosionHandler.AddExplosion(x, y, true);
						m_energySource.energy -= 0.08;
						m_main.shipExplodeSnd.play();
					}
				}
				else
				{
					dx /= d;
					dy /= d;
					x -= m_speed*dx;
					y -= m_speed*dy;
				}
			}
		}		
	}
	
	public function new(px:Float, py:Float, speed:Float, et:EnemyTypes, main:Main) 
	{
		super();
		if (et == EnemyTypes.StraightAttackAndExplodeAtEnergySource)
		{
			m_energySource = main.findClosestEnergySource(px, py);
		}
		
		m_main = main;
		x = px;
		y = py;
		m_shotPulse = 100;
		m_speed = speed;
		m_enemyType = et;
		
		if (et == EnemyTypes.StraightAttackAndExplodeAtEnergySource)
		{
			bmp = new Bitmap(Assets.getBitmapData("img/enemy.png"));
			energy = 0.2;
		}
		if (et == EnemyTypes.FireAtTower)
		{
			bmp = new Bitmap(Assets.getBitmapData("img/enemy2s.png"));
			energy = 2.0;
		}
		if (et == EnemyTypes.StraightAttackAndExplodeAtTower)
		{
			bmp = new Bitmap(Assets.getBitmapData("img/enemy3s.png"));
			energy = 0.5;
		}
		if (et == EnemyTypes.FireAtEnergySource)
		{
			bmp = new Bitmap(Assets.getBitmapData("img/enemy4s.png"));
			energy = 1.0;
		}
			
		m_lighting = new Bitmap(Assets.getBitmapData("img/explosion.png"));
		m_lighting.x = -1*m_lighting.width / 2;
		m_lighting.y = -1*m_lighting.height / 2;
		m_lighting.scaleX = 1;
		m_lighting.scaleY = 1;
		m_lighting.blendMode = BlendMode.HARDLIGHT;
		addChild(m_lighting);
		bmp.x = -bmp.width / 2;
		bmp.y = -bmp.height / 2;
		m_sprite = new Sprite();
		addChild(m_sprite);
		m_sprite.addChild(bmp);
	}
}